/*
 Licence GPL-V3
 alex.nouvel@free.fr - 14sept2018
 Load, resize, rotate and save image
*/

#include <gtk/gtk.h>
#include <wand/MagickWand.h>
#include <string.h>
#include <unistd.h>

#define PATH_CONFIG "/usr/share/convertImage4web/"

typedef struct {
	GtkWidget *win;
	GtkWidget *labelPath;
	GtkWidget *labelImage;
	GtkWidget *labelNewImage;
	GtkWidget *entry;
	GtkWidget *image;
	MagickWand *m_wand;
	DrawingWand *d_wand;
	char *filename;
	gchar *shortname;
	const gchar *dirTmp;
	int width;
	int height;
}Data;

void convert_image(GtkWidget *button, Data *data);

void info_image(Data *data) {	
//	m_wand = NewMagickWand();
	MagickReadImage(data->m_wand, data->filename);
	data->width = MagickGetImageWidth(data->m_wand);
	data->height = MagickGetImageHeight(data->m_wand);
	printf("Info Image\n Nom : %s\nLongueur : %d\nHauteur : %d\n", data->filename, data->width, data->height);
	// convert Int to String with glib's function
	gchar *str_width = g_strdup_printf("%i", data->width);
	gchar *str_height = g_strdup_printf("%i", data->height);
	const gchar *str_path = g_strconcat("Chemin de l'image :\n", data->filename, NULL);
	const gchar *str_info = g_strconcat("Longueur : ",  str_width, 
			"px\nHauteur : ", str_height, 
			"px\n\nChoisissez une longueur (en pixel)\n puis cliquez sur Convertir", NULL);
	gtk_label_set_text(GTK_LABEL(data->labelPath),  str_path);
	gtk_label_set_text(GTK_LABEL(data->labelImage), str_info);
}

static char *get_shortname(char *path_file) {
	int i=0;
	gchar **tab_str = g_strsplit(path_file, "/", -1);
	while (tab_str[i] != NULL)  i++;
	
	return tab_str[i-1];
}

static void update_preview (GtkFileChooser *chooserPreview, gpointer data) {
	GtkWidget *preview;
	char *filename;
	GdkPixbuf *pixbuf;
	gboolean have_preview;
	
	preview = GTK_WIDGET (data);
	filename = gtk_file_chooser_get_preview_filename (chooserPreview);
	
	pixbuf = gdk_pixbuf_new_from_file_at_size (filename, 192, 192, NULL);
	have_preview = (pixbuf != NULL);
	g_free (filename);
	
	gtk_image_set_from_pixbuf (GTK_IMAGE (preview), pixbuf);
	if (pixbuf)  g_object_unref (pixbuf);
	
	gtk_file_chooser_set_preview_widget_active (chooserPreview, have_preview);
}


void open_image(GtkWidget *menuItemOpen, Data *data) {
	GtkWidget *dialog;
	GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_OPEN;
	gint res;
	
	dialog = gtk_file_chooser_dialog_new("Ouverture d'une image",
		GTK_WINDOW(data->win), action,
		("_Annuler"), GTK_RESPONSE_CANCEL,
		("_Ouvrir"), GTK_RESPONSE_ACCEPT,
		NULL);
	
	
	GtkWidget *preview;
	
	GtkFileChooser *chooserPreview = GTK_FILE_CHOOSER(dialog);
	preview = gtk_image_new();
	gtk_file_chooser_set_preview_widget (chooserPreview, preview);
	g_signal_connect (chooserPreview, "update-preview", 
			G_CALLBACK (update_preview), preview);
	
	
	res = gtk_dialog_run(GTK_DIALOG (dialog));
	if (res == GTK_RESPONSE_ACCEPT) {
		GtkFileChooser *chooser = GTK_FILE_CHOOSER(dialog);
		data->filename = gtk_file_chooser_get_filename (chooser);
		printf("fichier %s\n", data->filename);
		gchar **tab_str = g_strsplit(data->filename, "/", -1);
	
		int i=0;
		while ( tab_str[i] != NULL ) {
			printf("valeur : %s\n", tab_str[i]);
			i++;
		}
		data->shortname = tab_str[i-1];
		printf("Nom shortname: %s\n", data->shortname);
		printf("test : 0 %s \t 1 %s \t 3 %s\n", tab_str[0], tab_str[1], tab_str[3]);
		info_image(data);
		gtk_image_set_from_file(GTK_IMAGE(data->image), data->filename);
	}
	
	gtk_widget_destroy(dialog);
}

void change_cursor(GtkWidget *button, Data *data, GdkCursorType cursor) {
	GdkWindow *gdk_win = gtk_widget_get_window(GTK_WIDGET(data->win));
	GdkDisplay *display = gdk_display_get_default();
	gdk_window_set_cursor(gdk_win, gdk_cursor_new_for_display(display, cursor));
	while (gtk_events_pending()) gtk_main_iteration();
}

void convert_image(GtkWidget *button, Data *data)  {
	printf("Conversion de taille\n");
	MagickBooleanType res = MagickFalse;
	
	if ( data->filename != NULL ) {
		printf("L'image existe\n");
		res = MagickReadImage(data->m_wand, data->filename);
	} else {
		printf("Pas d'image\n");
		res = MagickFalse;
	}
	
	if ( res == MagickTrue ) {
		change_cursor(button, data, GDK_WATCH);
		printf("short_name = %s\n", data->shortname);
		float ratio;
		const gchar *text_entry = gtk_entry_get_text(GTK_ENTRY(data->entry));
		int new_width = atoi(text_entry);
		int new_height;
		ratio = ( ((float) data->width) / ((float) new_width) );
		printf("Avant conversion\n Taille : %d \t hauteur : %d\nratio = %f\n", data->width, data->height, ratio);

		new_height = (int) ( (  (float) data->height ) / ratio ); 

		printf("Apres conversion\n Taille : %d \t hauteur : %d\n", new_width, new_height);
		
	//	MagickReadImage(data->m_wand, data->filename);
		MagickResizeImage(data->m_wand, new_width, new_height, LanczosFilter, 1);
		printf("Appel de g_strconcat dans convert_image\n");
		printf("short_name = %s\n", data->shortname);

		gchar *new_name = g_strconcat("resize", data->shortname, NULL);
		gchar *new_path = g_build_filename(data->dirTmp, new_name, NULL);
		printf("Nouveau nom : %s\n", new_name);
		printf("Nouveau chemin : %s\n", new_path);
		if ( MagickWriteImage(data->m_wand, new_path) ) {
			printf("Ecriture de l'image et chargement\n");
			gtk_image_set_from_file(GTK_IMAGE(data->image), new_path);
			gchar *str_new_width = g_strdup_printf("%i", new_width);
			gchar *str_new_height = g_strdup_printf("%i", new_height);
			gchar *str_label = g_strconcat("Taille apres conversion :\nLongueur ", str_new_width, "\nHauteur : ", str_new_height, NULL);
			gtk_label_set_text(GTK_LABEL(data->labelNewImage),  str_label);
			data->filename = new_path;
			gchar *str_width = g_strdup_printf("%i", data->width);
			gchar *str_height = g_strdup_printf("%i", data->height);
			const gchar *str_info = g_strconcat("Taille d'origine :\nLongueur : ",  str_width, 
				"px\nHauteur : ", str_height, NULL);
			gtk_label_set_text(GTK_LABEL(data->labelImage), str_info);
	//			gdk_window_set_cursor(gdk_win, gdk_cursor_new_for_display(display, GDK_ARROW));

		}
				change_cursor(button, data, GDK_LEFT_PTR);

	}//end ReadImage MagickTrue

/*	
	// clean up
	if (data->m_wand) data->m_wand = DestroyMagickWand(data->m_wand);

	MagickWandTerminus();
	*/
}

void rotate_image(GtkWidget *buttonRotate, Data *data) {
	printf("Rotation de l'image\n");
	MagickBooleanType res = MagickFalse;
	
	if ( data->filename != NULL ) {
		printf("L'image existe\n");
		res = MagickReadImage(data->m_wand, data->filename);
	} else {
		printf("Pas d'image\n");
		res = MagickFalse;
	}
	
	if ( res == MagickTrue) {
		PixelWand *background;
		background = NewPixelWand();
		PixelSetColor(background, "#000000");
		MagickRotateImage(data->m_wand, background, -90);
		int tmp_width = data->width;
		data->width = data->height;
		data->height = tmp_width;
		DestroyPixelWand(background);
		gchar *new_name = g_strconcat("rotate", data->shortname, NULL);
		gchar *new_path = g_build_filename(data->dirTmp, new_name, NULL);
			printf("Nouveau chemin : %s\n", new_path);

		if (MagickWriteImage(data->m_wand, new_path) ) {
			printf("Ecriture de l'image dans Rotation\n");
			gtk_image_set_from_file(GTK_IMAGE(data->image), new_path);
			data->filename = new_path;
		}
	} //end if ReadImage
}

void saveas_image(GtkWidget *menuItemSaveAs, Data *data) {
	GtkWidget *dialog;
	GtkFileChooser *chooser;
	GtkFileChooserAction action = GTK_FILE_CHOOSER_ACTION_SAVE;
	gint res;
	
	dialog = gtk_file_chooser_dialog_new("Enregistrer Sous",
		GTK_WINDOW(data->win), action,
		("_Cancel"), GTK_RESPONSE_CANCEL,
		("_Save"), GTK_RESPONSE_ACCEPT,
		NULL);

	chooser = GTK_FILE_CHOOSER (dialog);
	gtk_file_chooser_set_do_overwrite_confirmation (chooser, TRUE);
	
//		gchar *new_name = g_strconcat("resize", data->shortname, NULL);

//	gtk_file_chooser_set_current_name(chooser, new_name);
	data->shortname = get_shortname(data->filename);
		gtk_file_chooser_set_current_name(chooser, data->shortname);

	
	res = gtk_dialog_run (GTK_DIALOG (dialog));
	if (res == GTK_RESPONSE_ACCEPT) {
		char *filename;
		filename = gtk_file_chooser_get_filename(chooser);
		MagickWriteImage(data->m_wand, filename);
		g_free(filename);
	}
	
	gtk_widget_destroy(dialog);
}

void display_about(GtkWidget *menuItemAbout, Data *data) {
	GtkWidget *dialog, *label, *content_area;
	GtkDialogFlags flags;
	
	flags = GTK_DIALOG_DESTROY_WITH_PARENT;
	dialog = gtk_dialog_new_with_buttons("Aide", GTK_WINDOW(data->win), 
		flags, ("OK"), GTK_RESPONSE_NONE,
		NULL);
	const gchar *str_message = g_strconcat("Simple logiciel pour convertir la taille d'une photo\n",
		"au format web.\n",
		"Licence libre GPLV-3\n",
		"Pour obtenir les sources, contacter l'auteur :\n",
		"alex.nouvel@free.fr",
		NULL);
	label = gtk_label_new(str_message);
	content_area = gtk_dialog_get_content_area (GTK_DIALOG (dialog));
	
	g_signal_connect_swapped (dialog, "response", G_CALLBACK (gtk_widget_destroy), dialog);
	
	gtk_container_add (GTK_CONTAINER (content_area), label);
	gtk_widget_show_all (dialog);
}

int main(int argc, char **argv)
{
 Data data;

 GtkWidget *menuBar;
 GtkWidget *menu;
 GtkWidget *menuItemFile;
 GtkWidget *menuItemOpen;
 GtkWidget *menuItemSaveAs;
 GtkWidget *menuItemQuit;
	
	GtkWidget *menuHelp;
	GtkWidget *menuItemHelp;
	GtkWidget *menuItemAbout;
	
	GtkWidget *scrolledImage;

	GtkWidget *mainBox;
	GtkWidget *boxLeftRight;
	GtkWidget *boxRight;
	GtkWidget *boxLeft;
	
 GtkWidget *button;
 GtkWidget *buttonRotate;

/* ON INITIALISE GTK */
 gtk_init(&argc, &argv);

/* Create window */
 data.win = gtk_window_new(GTK_WINDOW_TOPLEVEL);
 data.filename = NULL;

/* Init Structure */
	data.m_wand = NULL;
	data.m_wand = NewMagickWand();
	data.d_wand = NULL;
	data.d_wand = NewDrawingWand();
	data.dirTmp = g_get_tmp_dir();
printf("rep tmp : %s\n", data.dirTmp);
	
/* Create box */ 
 mainBox = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
 boxLeftRight = gtk_box_new(GTK_ORIENTATION_HORIZONTAL, 2);
 boxRight = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);
 boxLeft = gtk_box_new(GTK_ORIENTATION_VERTICAL, 2);

/* Create menu */
 menuBar = gtk_menu_bar_new();
 menuItemFile = gtk_menu_item_new_with_mnemonic("_Fichier");
 menu = gtk_menu_new();
 menuItemOpen = gtk_menu_item_new_with_label("Ouvrir");
 menuItemSaveAs = gtk_menu_item_new_with_label("Enregistrer Sous ...");
 menuItemQuit = gtk_menu_item_new_with_label("Quitter");

 gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItemOpen);
 gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItemSaveAs);
 gtk_menu_shell_append(GTK_MENU_SHELL(menu), menuItemQuit);
 gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItemFile), menu);
 gtk_menu_shell_append(GTK_MENU_SHELL(menuBar), menuItemFile);
 
 menuItemHelp = gtk_menu_item_new_with_mnemonic("Aide");
 menuHelp = gtk_menu_new();
 menuItemAbout = gtk_menu_item_new_with_label("A propos");
 
 gtk_menu_shell_append(GTK_MENU_SHELL(menuHelp), menuItemAbout);
 gtk_menu_item_set_submenu(GTK_MENU_ITEM(menuItemHelp), menuHelp);
 gtk_menu_shell_append(GTK_MENU_SHELL(menuBar), menuItemHelp);

 data.labelPath = gtk_label_new("Aucune image selectionnee");
 data.labelImage = gtk_label_new("Aucune caracteristique");
 data.labelNewImage = gtk_label_new("En attente de conversion");
 scrolledImage = gtk_scrolled_window_new (NULL, NULL);
	const gchar *str_path_empty = g_strconcat(PATH_CONFIG, "images/empty.jpeg", NULL);
 	data.image = gtk_image_new_from_file(str_path_empty);
	data.entry = gtk_entry_new();
	gtk_entry_set_text(GTK_ENTRY(data.entry), "500");
	button = gtk_button_new_with_label("Convertir");
//	const gchar *str_path_icon = g_strconcat(PATH_CONFIG, "", NULL);
//	printf("chemin icone : %s\n", str_path_icon);
//	buttonRotate = gtk_button_new_from_icon_name("object-rotate-left", GTK_ICON_SIZE_BUTTON);

	buttonRotate = gtk_button_new();
	const gchar *str_path_rotate = g_strconcat(PATH_CONFIG, "images/rotate90.png", NULL);
	GtkWidget *im = gtk_image_new_from_file(str_path_rotate);
	gtk_button_set_image(GTK_BUTTON(buttonRotate), (im) );

/* Parametres des objets (ici la taille de la fenetre) */
 gtk_window_set_default_size(GTK_WINDOW(data.win), 700, 600);
 gtk_window_set_title(GTK_WINDOW(data.win), "convertImage4web");

/* Placement des objets dans la fenetre principal et dans la boite */
 gtk_container_add(GTK_CONTAINER(data.win), mainBox);
 gtk_box_pack_start(GTK_BOX(mainBox), menuBar, FALSE, TRUE, 0);
 gtk_box_pack_start(GTK_BOX(mainBox), boxLeftRight, TRUE, TRUE, 0);
 gtk_box_pack_start(GTK_BOX(boxLeftRight), boxLeft, FALSE, FALSE, 10);
 gtk_box_pack_start(GTK_BOX(boxLeftRight), boxRight, TRUE, TRUE, 0);
 
 gtk_box_pack_start(GTK_BOX(boxLeft), data.labelImage, FALSE, FALSE, 0);
 gtk_box_pack_start(GTK_BOX(boxLeft), data.labelNewImage, FALSE, FALSE, 0);
 gtk_box_pack_start(GTK_BOX(boxLeft), data.entry, FALSE, FALSE, 10);
 gtk_box_pack_start(GTK_BOX(boxLeft), button, FALSE, FALSE, 10);
 gtk_box_pack_start(GTK_BOX(boxLeft), buttonRotate, FALSE, FALSE, 10);
// gtk_widget_set_size_request (data.entry, 50, -1);
 //gtk_box_set_homogeneous(GTK_BOX(boxLeft), FALSE);
 
 gtk_box_pack_start(GTK_BOX(boxRight), data.labelPath, FALSE, TRUE, 0);
   gtk_box_pack_start(GTK_BOX(boxRight), scrolledImage, TRUE, TRUE, 0);
   gtk_container_add(GTK_CONTAINER(scrolledImage), data.image);



// SIGNAL MENU
g_signal_connect(G_OBJECT(menuItemOpen), "activate", G_CALLBACK(open_image), (gpointer) &data);
g_signal_connect(G_OBJECT(menuItemSaveAs), "activate", G_CALLBACK(saveas_image), (gpointer) &data);
g_signal_connect(G_OBJECT(menuItemQuit), "activate", G_CALLBACK(gtk_main_quit), NULL);

g_signal_connect(G_OBJECT(menuItemAbout), "activate", G_CALLBACK(display_about), (gpointer) &data);

// SIGNAL DU CLIQUE DU BOUTON
g_signal_connect(G_OBJECT(button), "clicked", G_CALLBACK(convert_image), (gpointer) &data);
g_signal_connect(G_OBJECT(buttonRotate), "clicked", G_CALLBACK(rotate_image), (gpointer) &data);

// SIGNAL POUR FERMER LA FENeTRE
 g_signal_connect(G_OBJECT(data.win), "destroy", G_CALLBACK( gtk_main_quit ), NULL);

// ON AFFICHE LA FENETRE
 gtk_widget_show_all(data.win);

// BOUCLE D'ATTENTE D'EVENEMENT
 gtk_main();
 return 0;
}

